#ifndef SHARE_WINDOW_H
#define SHARE_WINDOW_H

#include <app/Message.h>
#include <interface/Window.h>
#include <interface/TextControl.h>
#include <interface/TextView.h>
#include <storage/Directory.h>

#include "queue/PortableQueue.h"
#include "message/PortableMessage.h"
#include "besupport/MessageTransceiverThread.h"

#include "PrefilledBitmap.h"
#include "CLVListItem.h"

#include "ShareConstants.h"

class ShareNetClient;
class ShareFileTransfer;
class RemoteUserItem;
class RemoteFileItem;
class ShareColumn;
class ShareMIMEInfo;
class ColumnListView;

class ShareWindow : public BWindow
{
public:
   ShareWindow();
   ~ShareWindow();

   virtual void MessageReceived(BMessage * msg);

   virtual bool QuitRequested();

   void SetConnectStatus(bool isConnecting, bool isConnected);
   void LogMessage(LogMessageType type, const char * text, const char * optSessionID=NULL);

   void PutUser(const char * sessionID, const char * userName, const char * hostName, int userPort);
   void RemoveUser(const char * sessionID);
   
   void PutResult(const char * sessionID, const char * fileName, PortableMessageRef fileInfo);
   void RemoveResult(const char * sessionID, const char * fileName);

   void FileTransferConnected(ShareFileTransfer * who);
   void FileTransferDisconnected(ShareFileTransfer * who);

   // Called by a ShareFileTransfer who wants his GUI updated.
   void RefreshTransferItem(ShareFileTransfer * item);

   // Returns a bitmap representing the icon of the given type, if any
   const BBitmap * GetBitmap(const char * mimeType);
 
   // Given a session ID, returns the user name for that ID.
   // If the session ID isn't known, "<unknown>" is returned.
   const char * GetUserNameForSession(const char * sessionID) const;

private:
   static int CompareFunc(const CLVListItem* item1, const CLVListItem* item2, int32 sort_key);
   int Compare(const RemoteFileItem * rf1, const RemoteFileItem * rf2, int32 sort_key) const;

   friend class RemoteUserItem; 
   friend class RemoteFileItem;
   friend class ShareColumn;
   friend class ShareMIMEInfo;

   // called by RemoteUserItem
   void AddFileItem(RemoteFileItem * item);
   void RemoveFileItem(RemoteFileItem * item);
   void RefreshFileItem(RemoteFileItem * item);
   void RefreshUserItem(RemoteUserItem * item);

   const char * GetFileCellText(const RemoteFileItem * item, int32 columnIndex) const;
   const BBitmap * GetBitmap(const RemoteFileItem * file, int32 columnIndex) const;

   // 'what' codes that are used internally by this class
   enum 
   {
      SHAREWINDOW_COMMAND_RECONNECT_TO_SERVER = '$win',
      SHAREWINDOW_COMMAND_DISCONNECT_FROM_SERVER,
      SHAREWINDOW_COMMAND_ABOUT,
      SHAREWINDOW_COMMAND_SEND_CHAT_TEXT,
      SHAREWINDOW_COMMAND_CHANGE_FILE_NAME_QUERY,
      SHAREWINDOW_COMMAND_USER_CHANGED_NAME,
      SHAREWINDOW_COMMAND_USER_CHANGED_SERVER,
      SHAREWINDOW_COMMAND_ENABLE_QUERY,
      SHAREWINDOW_COMMAND_DISABLE_QUERY,
      SHAREWINDOW_COMMAND_TOGGLE_COLUMN,
      SHAREWINDOW_COMMAND_BEGIN_DOWNLOADS,
      SHAREWINDOW_COMMAND_CANCEL_DOWNLOADS,
      SHAREWINDOW_COMMAND_CLEAR_FINISHED_DOWNLOADS,
      SHAREWINDOW_COMMAND_RESULT_SELECTION_CHANGED,
      SHAREWINDOW_COMMAND_REMOVE_SESSION,
      SHAREWINDOW_COMMAND_NEW_PEER_SESSION
   };

   status_t GetAppSubdir(const char * subDirName, BDirectory & setSubDirRef, bool createIfNecessary) const;
   void UpdateConnectStatus(bool updateTitleToo);
   void UpdateQueryEnabledStatus();
   void SetQueryEnabled(bool enabled);

   void ClearUsers();   // also clears results, as results are held by users
   void ClearResults();

   void CreateColumn(ShareMIMEInfo * optMIMEInfo, const char * columnName, bool remote);
   ShareMIMEInfo * CacheMIMETypeInfo(const char * mimeString);

   void UpdateDownloadButtonStatus();
   void RequestDownloads();

   BDirectory _shareDir;
   BDirectory _downloadsDir;

   bool _queryEnabled;

   bool _isConnecting;
   bool _isConnected;
   PortableString _connectedTo;

   BButton * _enableQueryButton;
   BButton * _disableQueryButton;

   BButton * _clearFinishedDownloadsButton;
   BButton * _requestDownloadsButton;
   BButton * _cancelTransfersButton;

   BTextControl * _serverEntry;
   BTextControl * _userNameEntry;

   BTextControl * _fileNameQueryEntry;

   BTextView * _chatText;
   BTextControl * _textEntry;

   BMenuBar * _menuBar;

   BMenuItem * _connectMenuItem;
   BMenuItem * _disconnectMenuItem;
 
   BMenu * _attribMenu;

   ColumnListView * _resultsView;
   ColumnListView * _usersView;

   BListView * _transferList;

   ShareNetClient * _netClient;

   PrefilledBitmap _defaultBitmap;

   PortableHashtable<const char *, RemoteUserItem *> _users;

   PortableHashtable<PortableString, float> _activeAttribs;        /* attribs the user wants to see, and their widths */
   PortableHashtable<const char *, ShareColumn *> _columns;        /* columns that are available */
   PortableHashtable<const char *, BMenuItem *> _attribMenuItems;  /* toggles for available columns */
   PortableHashtable<const char *, ShareMIMEInfo *> _mimeInfos;    /* cached MIME information */

   MessageTransceiverThread * _shareServer;  // used to accept incoming connections from peers
};

#endif
